@@ -62,12 +62,30 @@ def define_callbacks(lr_scheduler = None):
 	if len(list(callbacks.keys())) == 0:
 		print("Error! No callbacks defined but callback called")
 
+## Worked, integrate later
+def check_arguments(func):
+	flag = True
+	def inner(*args, **kwargs):
+		
+		for arg in args:
+			print(arg)
+		
+		for arg in kwargs:
+			print(arg)
+					
+		if flag:
+			func(*args, **kwargs)	
+		else:
+			print("Error! Check your arguments format! Define the arguments as per the instructions..")
+	
+	return inner
 
-def define_augmentations(flip_left_right = False, hue = None, contrast = None, brightness = None):
+
+#@check_arguments
+def define_augmentations(flip_left_right = False, hue = None, contrast = None, brightness = None, random_crop = None, random_saturation = None,random_zoom = None, flip_up_down = False, random_rotation = None, random_shear = None, random_shift = None):
 	
 	
 	global augments
-	## add random saturation, random crop, zoom, shearing, rotation etc
 	
 	augments = {}
 
@@ -77,17 +95,36 @@ def define_augmentations(flip_left_right = False, hue = None, contrast = None, b
 	if hue:
 		augments['random_hue'] = hue
 
-
 	if contrast:
-		if isinstance(contrast, tuple):
-			augments['random_contrast'] = contrast
+		augments['random_contrast'] = contrast
 
 	if brightness:
 		augments['brightness'] = brightness
+	
+	if random_crop:
+		augments['random_crop'] = random_crop
+	
+	if random_saturation:
+		augments['random_saturation'] = random_saturation
+	
+	if random_zoom:
+		augments['random_zoom'] = random_zoom
+		
+	if flip_up_down:
+		augments['flip_up_down'] = True
+	
+	if random_rotation:
+		augments['random_rotation'] = random_rotation
+			
+	if random_shear:
+		augments['random_shear'] = random_shear
+	
+	if random_shift:
+		augments['random_shift'] = random_shift
 
-
+	print("Defined augmentes -> \n", augments, "\n")
 	if len(list(augments.items())) == 0:
-		print("Error! No augments defined...")
+		print("Error! Define augments called but no augments defined...")
 		return
 
 
@@ -99,25 +136,42 @@ def augmentations(image, label):
 	for aug in augments.items():
 		if aug[0] == 'flip_left_right' and aug[1] == True:
 			image = tf.image.random_flip_left_right(image)
-		if aug[0] == 'hue':
+		if aug[0] == 'random_hue':
 			image = tf.image.random_hue(image, aug[1])
-		if aug[0] == 'contrast':
+		if aug[0] == 'random_contrast':
 			image = tf.image.random_contrast(image, aug[1][0], aug[1][1])
 		if aug[0] == 'brightness':
 			image = tf.image.random_brightness(image, aug[1])
+		if aug[0] == 'random_crop':
+			image = tf.image.random_crop(image, [aug[1][0], aug[1][1],aug[1][2]])
+		if aug[0] == 'random_saturation':
+			image = tf.image.random_saturation(image, aug[1][0], aug[1][1])
+		if aug[0] == 'random_zoom':
+			image = tf.keras.preprocessing.image.random_zoom(image, (aug[1][0], aug[1][1]))
+		if aug[0] == 'flip_up_down' and aug[1] == True:
+			image = tf.image.random_flip_up_down(image)
+		if aug[0] == 'random_rotation':
+			image = tf.keras.preprocessing.image.random_rotation(image, aug[1])
+		if aug[0] == 'random_shear':
+			image = tf.keras.preprocessing.image.random_shear(image, aug[1])
+		if aug[0] == 'random_shift':
+			image = tf.keras.preprocessing.image.random_shift(image, aug[1][0], aug[1][1])
+		
 
 	return image, label
 
 
 
 
-def augment_and_training_report(models_list, GCS_DS_PATH, train_tfrec_path, val_tfrec_path, batch_size, tpu, n_class, steps_per_epoch, epochs, classification_model = 'default', freeze = False, input_shape = [512,512,3], activation = 'softmax', weights = "imagenet", optimizer = "adam", loss = "sparse_categorical_crossentropy", metrics = "sparse_categorical_accuracy", plot = False):
-
-
+def get_models_training_report(models,tpu, n_class, GCS_DS_PATH, train_tfrec_path, steps_per_epoch, epochs, batch_size, val_tfrec_path ,  classification_model = 'default', freeze = False, input_shape = [512,512,3], activation = 'softmax', weights = "imagenet", optimizer = "adam", loss = "sparse_categorical_crossentropy", metrics = "sparse_categorical_accuracy", plot = False):
+	
 	import pandas as pd
 	import numpy as np
 	import gc
 	from quick_ml.load_models_quick import create_model
+	from quick_ml.begin_tpu import get_training_dataset
+	from quick_ml.begin_tpu import get_validation_dataset
+	
 	print("Num classes => " , n_class)
 	print("\n\n", '#'*90, '\n\n')
 	print("\n\n TO OBTAIN THE BEST PERFORMANCE, PLEASE USE THE PRETRAINED MODEL WEIGHTS AS INPUT. Dataset Link -> https://www.kaggle.com/superficiallybot/tf-keras-pretrained-weights\n\n")
@@ -126,6 +180,7 @@ def augment_and_training_report(models_list, GCS_DS_PATH, train_tfrec_path, val_
 	df = pd.DataFrame(columns = ['Model_Name', 'Accuracy_top1', 'Accuracy_top3', "Val_Accuracy_top1", "Val_Accuracy_top3"])
 
 	for m in models:
+		overall = {}
 		print(f"Beginning with model -> {m}")
 		tf.tpu.experimental.initialize_tpu_system(tpu)
 		strategy = tf.distribute.experimental.TPUStrategy(tpu)
@@ -135,15 +190,37 @@ def augment_and_training_report(models_list, GCS_DS_PATH, train_tfrec_path, val_
 			else:
 				model = create_model( freeze = freeze, input_shape = input_shape, activation = activation, weights = weights, optimizer = optimizer, loss = loss, metrics = metrics , classes = n_class, model_name = m)
 				
-		from quick_ml.begin_tpu import get_training_dataset, get_validation_dataset
-		history = model.fit(get_training_dataset(GCS_DS_PATH, train_tfrec_path, batch_size, augmentation = True), get_validation_dataset(GCS_DS_PATH, val_tfrec_path, batch_size, augmentation = True), steps_per_epoch = steps_per_epoch, epochs = epochs,batch_size = batch_size, validation_data =  val_data, callbacks = callbacks, verbose = 0)
+		for epoch in range(epochs):
+			if epoch == 0:
+				print(f"\nEpoch # {epoch + 1}\n")
+				print(f"\nCurrent Learning Rate -> {K.eval(model.optimizer.lr)}")
+				history = model.fit(get_training_dataset(GCS_DS_PATH, train_tfrec_path, batch_size, augmentation = True), validation_data = get_validation_dataset(GCS_DS_PATH, val_tfrec_path, batch_size, augmentation = True), epochs = 1, steps_per_epoch = steps_per_epoch, batch_size = batch_size, verbose = 1)
+				if callbacks is not None:
+					set_learning_rate(model, epoch + 1)
+				
+				for k,v in history.history.items():
+					overall[k] = v
+			
+			else:
+				print(f"\nEpoch # {epoch + 1}\n")
+				print(f"\nCurrent Learning Rate -> {K.eval(model.optimizer.lr)}")
+				
+				history = model.fit(get_training_dataset(GCS_DS_PATH, train_tfrec_path, batch_size, augmentation = True), validation_data = get_validation_dataset(GCS_DS_PATH, val_tfrec_path, batch_size, augmentation = True), epochs = 1, steps_per_epoch = steps_per_epoch, batch_size = batch_size, verbose = 0)
+				
+				if callbacks is not None:
+					set_learning_rate(model, epoch + 1)
+			
+				for k,v in history.history.items():
+					overall[k].extend(v)
+				
+				
+				
 		tf.keras.backend.clear_session()
 		tf.compat.v1.reset_default_graph()
 		del model
 		gc.collect()
 		
-
-		df = df.append(pd.DataFrame([[m, history.history[metrics][-1], np.mean(history.history[metrics][-3:]) , history.history['val_' + metrics][-1], np.mean(history.history['val_' + metrics][-3:])]], columns = ['Model_Name', 'Accuracy_top1', 'Accuracy_top3', "Val_Accuracy_top1", "Val_Accuracy_top3"]), ignore_index = True)
+		df = df.append(pd.DataFrame([[m, overall[metrics][-1], np.mean(overall[metrics][-3:]) , overall['val_' + metrics][-1], np.mean(overall['val_' + metrics][-3:])]], columns = ['Model_Name', 'Accuracy_top1', 'Accuracy_top3', "Val_Accuracy_top1", "Val_Accuracy_top3"]), ignore_index = True)
 		print(f"Done with model -> {m}")
 
 	if plot:
@@ -164,7 +241,6 @@ def augment_and_training_report(models_list, GCS_DS_PATH, train_tfrec_path, val_
 
 
 
-
 def set_learning_rate(model, epoch):
 	
 	global callbacks
@@ -238,7 +314,7 @@ def augment_and_train(model, GCS_DS_PATH, train_tfrec_path, val_tfrec_path, batc
 		plt.show()
 
 
-    
+	
 
 
 
